﻿package cj.components.scrollbar {
	
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.events.MouseEvent;
	import flash.events.Event;
	
	import com.greensock.TweenLite;
	import com.greensock.easing.Quint;
	
	import cj.components.scrollbar.events.ScrollArrowEvent;
	
	public final class ScrolBar extends Sprite {
		
		private var barHeight:int;
		private var top:Shape;
		private var bottom:Shape;
		private var bar1:Shape;
		private var bar2:Shape;
		private var line:Shape;
		private var mid:Shape;
		private var dragger:Sprite;
		private var holder:Sprite;
		private var dragHolder:Sprite;
		private var topA:ScrollArrow;
		private var bottomA:ScrollArrow;
		
		private var max:int;
		private var min:int;
		private var storeH:int;
		private var distH:int;
		private var where:Number;
		private var go:Number;
		private var difY:Number;
		private var dragColor:uint;
		private var lineColor:uint;
		
		public function ScrolBar(){};
		
		public function activate(numItems:int, high:int, hold:Sprite, dColor:uint, lColor:uint) {
			
			barHeight = high / numItems;
			(barHeight < 30) ? barHeight = 30 : null;
			
			holder = hold;
			distH = high;
			
			dragColor = dColor;
			lineColor = lColor;
			
			topA = new ScrollArrow(true);
			bottomA = new ScrollArrow();

			min = topA.height;
			high -= topA.height * 2;
			
			line = drawShape(1, high, lineColor);
			
			var half:int = barHeight >> 1;
			
			top = drawShape(5, half, dragColor);
			bottom = drawShape(5, half, dragColor);
			bar1 = drawShape(5, 1, dragColor);
			bar2 = drawShape(5, 1, dragColor);
			mid = drawShape(5, 5, lineColor);
			
			mid.y = half;
			bar1.y = half + 1;
			bar2.y = bar1.y + 2;
			bottom.y = bar2.y + 2;
			
			bottomA.rotation = 180;
			bottomA.x = bottomA.width - 6;
			bottomA.y = min + high + bottomA.height;
			
			dragger = new Sprite();
			dragHolder = new Sprite();
			dragHolder.mouseEnabled = false;
			
			dragger.addChild(top);
			dragger.addChild(bottom);
			dragger.addChild(mid);
			dragger.addChild(bar1);
			dragger.addChild(bar2);
			
			dragger.graphics.beginFill(0, 0);
			dragger.graphics.drawRect(-3, 0, 11, dragger.height);
			dragger.graphics.endFill();
			
			dragHolder.x = int((topA.width >> 1) - 5);
			dragHolder.y = min;
			
			dragHolder.addChild(dragger);
			
			line.x = dragHolder.x + 2;
			line.y = min;
			
			max = line.height - dragger.height;
			storeH = holder.height;
			
			addChild(line);
			addChild(dragHolder);
			addChild(topA);
			addChild(bottomA);
			
			addEventListener(Event.ADDED_TO_STAGE, added);
			
		}
		
		private function added(event:Event):void {
			
			removeEventListener(Event.ADDED_TO_STAGE, added);
			stage.addEventListener(MouseEvent.MOUSE_WHEEL, onMouse, false, 0, true);
			
			dragger.buttonMode = true;
			dragger.addEventListener(MouseEvent.MOUSE_DOWN, thumbDown, false, 0, true);
			addEventListener(ScrollArrowEvent.ARROW_CLICK, arrowClick, false, 0, true);
			
			addEventListener(Event.REMOVED_FROM_STAGE, removed, false, 0, true);
			
		}
		
		private function arrowClick(event:ScrollArrowEvent):void {
			
			if(event.isTop) {
				dragger.y -= 5;
			}
			else {
				dragger.y += 5;
			}
			
			scrollMe();
			
		}
		
		private function thumbDown(event:MouseEvent):void {
			
			stage.addEventListener(MouseEvent.MOUSE_MOVE, thumbMove, false, 0, true);
			stage.addEventListener(MouseEvent.MOUSE_UP, thumbUp, false, 0, true); 
			difY = this.mouseY - dragger.y;
			
		}
		
		private function thumbUp(event:MouseEvent):void {

			stage.removeEventListener(MouseEvent.MOUSE_MOVE, thumbMove);
			stage.removeEventListener(MouseEvent.MOUSE_UP, thumbUp); 
			
		}
		
		private function thumbMove(event:MouseEvent):void {
			
			dragger.y = this.mouseY - difY;
			scrollMe();
			
		}
		
		private function onMouse(event:MouseEvent):void {
			
			dragger.y -= event.delta * 2;
			scrollMe();
			
		}
		
		private function scrollMe():void {
			
			if(dragger.y <= 0) {
				
				dragger.y = 0;
				
			}
			else if(dragger.y >= max) {
				
				dragger.y = max;
				
			}
					
			where = dragger.y / max;
			
			go = -where * (storeH - distH);
			
			TweenLite.to(holder, 0.5, {y: go, ease: Quint.easeOut});
		
		}
		
		private function drawShape(w:int, h:int, col:uint):Shape {
			
			var sh:Shape = new Shape();
			sh.graphics.beginFill(col);
			sh.graphics.drawRect(0, 0, w, h);
			sh.graphics.endFill();
			
			return sh;
			
		}
		
		private function removed(event:Event):void {
			
			removeEventListener(Event.REMOVED_FROM_STAGE, removed);
			removeEventListener(Event.ADDED_TO_STAGE, added);
			removeEventListener(ScrollArrowEvent.ARROW_CLICK, arrowClick);
			dragger.removeEventListener(MouseEvent.MOUSE_DOWN, thumbDown);

			
			if(stage != null) {
				stage.removeEventListener(MouseEvent.MOUSE_WHEEL, onMouse);
				stage.removeEventListener(MouseEvent.MOUSE_MOVE, thumbMove);
				stage.removeEventListener(MouseEvent.MOUSE_UP, thumbUp); 
			}
			
			TweenLite.killTweensOf(holder);
			
			while(dragger.numChildren) {
				dragger.removeChildAt(0);
			}
			
			dragHolder.removeChildAt(0);
			
			while(this.numChildren) {
				removeChildAt(0);
			}
			
			dragger.graphics.clear();
			
			top.graphics.clear();
			bottom.graphics.clear();
			bar1.graphics.clear();
			bar2.graphics.clear();
			mid.graphics.clear();
			
			holder = null;
			top = null;
			bottom = null;
			bar1 = null;
			bar2 = null;
			line = null;
			mid = null;
			dragger = null;
			holder = null;
			dragHolder = null;
			topA = null;
			bottomA = null;
			
		}
		
	}
	
}










